@@ -74,20 +74,16 @@ class asio_connection
     friend class asio_connection_pool;
     friend class asio_client;
 public:
-    asio_connection(boost::asio::io_service& io_service, bool use_ssl, const std::function<void(boost::asio::ssl::context&)>& ssl_context_callback) :
+    asio_connection(boost::asio::io_service& io_service, bool start_with_ssl, const std::function<void(boost::asio::ssl::context&)>& ssl_context_callback) :
     m_socket(io_service),
+    m_ssl_context_callback(ssl_context_callback),
     m_pool_timer(io_service),
     m_is_reused(false),
     m_keep_alive(true)
     {
-        if (use_ssl)
+        if (start_with_ssl)
         {
-            boost::asio::ssl::context ssl_context(boost::asio::ssl::context::sslv23);
-            ssl_context.set_default_verify_paths();
-            ssl_context.set_options(boost::asio::ssl::context::default_workarounds);
-            ssl_context_callback(ssl_context);
-            m_ssl_stream = utility::details::make_unique<boost::asio::ssl::stream<boost::asio::ip::tcp::socket &>>(m_socket, ssl_context);
-
+            upgrade_to_ssl();
         }
     }
 
@@ -96,6 +92,18 @@ public:
         close();
     }
 
+    // This simply instantiates the internal state to support ssl. It does not perform the handshake.
+    void upgrade_to_ssl()
+    {
+        std::lock_guard<std::mutex> lock(m_socket_lock);
+        assert(!is_ssl());
+        boost::asio::ssl::context ssl_context(boost::asio::ssl::context::sslv23);
+        ssl_context.set_default_verify_paths();
+        ssl_context.set_options(boost::asio::ssl::context::default_workarounds);
+        m_ssl_context_callback(ssl_context);
+        m_ssl_stream = utility::details::make_unique<boost::asio::ssl::stream<boost::asio::ip::tcp::socket &>>(m_socket, ssl_context);
+    }
+
     void close()
     {
         std::lock_guard<std::mutex> lock(m_socket_lock);
@@ -136,6 +144,7 @@ public:
     template <typename HandshakeHandler, typename CertificateHandler>
     void async_handshake(boost::asio::ssl::stream_base::handshake_type type,
                          const http_client_config &config,
+                         const utility::string_t &host_name,
                          const HandshakeHandler &handshake_handler,
                          const CertificateHandler &cert_handler)
     {
@@ -152,6 +161,13 @@ public:
         {
             m_ssl_stream->set_verify_mode(boost::asio::ssl::context::verify_none);
         }
+
+        // Check to set host name for Server Name Indication (SNI)
+        if (config.is_tlsext_sni_enabled())
+        {
+            SSL_set_tlsext_host_name(m_ssl_stream->native_handle(), const_cast<char *>(host_name.data()));
+        }
+
         m_ssl_stream->async_handshake(type, handshake_handler);
     }
 
@@ -220,6 +236,8 @@ private:
     tcp::socket m_socket;
     std::unique_ptr<boost::asio::ssl::stream<tcp::socket &> > m_ssl_stream;
 
+    std::function<void(boost::asio::ssl::context&)> m_ssl_context_callback;
+
     boost::asio::deadline_timer m_pool_timer;
     bool m_is_reused;
     bool m_keep_alive;
@@ -229,10 +247,10 @@ class asio_connection_pool
 {
 public:
 
-    asio_connection_pool(boost::asio::io_service& io_service, bool use_ssl, const std::chrono::seconds &idle_timeout, const std::function<void(boost::asio::ssl::context&)> &ssl_context_callback) :
+    asio_connection_pool(boost::asio::io_service& io_service, bool start_with_ssl, const std::chrono::seconds &idle_timeout, const std::function<void(boost::asio::ssl::context&)> &ssl_context_callback) :
     m_io_service(io_service),
     m_timeout_secs(static_cast<int>(idle_timeout.count())),
-    m_use_ssl(use_ssl),
+    m_start_with_ssl(start_with_ssl),
     m_ssl_context_callback(ssl_context_callback)
     {}
 
@@ -269,7 +287,7 @@ public:
             lock.unlock();
 
             // No connections in pool => create a new connection instance.
-            return std::make_shared<asio_connection>(m_io_service, m_use_ssl, m_ssl_context_callback);
+            return std::make_shared<asio_connection>(m_io_service, m_start_with_ssl, m_ssl_context_callback);
         }
         else
         {
@@ -305,19 +323,21 @@ private:
 
     boost::asio::io_service& m_io_service;
     const int m_timeout_secs;
-    const bool m_use_ssl;
+    const bool m_start_with_ssl;
     const std::function<void(boost::asio::ssl::context&)>& m_ssl_context_callback;
     std::vector<std::shared_ptr<asio_connection> > m_connections;
     std::mutex m_connections_mutex;
 };
 
+
+
 class asio_client : public _http_client_communicator, public std::enable_shared_from_this<asio_client>
 {
 public:
     asio_client(http::uri address, http_client_config client_config)
     : _http_client_communicator(std::move(address), std::move(client_config))
     , m_pool(crossplat::threadpool::shared_instance().service(),
-             base_uri().scheme() == "https",
+             base_uri().scheme() == "https" && !_http_client_communicator::client_config().proxy().is_specified(),
              std::chrono::seconds(30), // Unused sockets are kept in pool for 30 seconds.
              this->client_config().get_ssl_context_callback()) 
     , m_resolver(crossplat::threadpool::shared_instance().service())
@@ -363,6 +383,173 @@ public:
         ctx->m_timer.set_ctx(std::weak_ptr<asio_context>(ctx));
         return ctx;
     }
+    
+    class ssl_proxy_tunnel : public std::enable_shared_from_this<ssl_proxy_tunnel>
+    {
+    public:
+        ssl_proxy_tunnel(std::shared_ptr<asio_context> context, std::function<void(std::shared_ptr<asio_context>)> ssl_tunnel_established)
+            : m_ssl_tunnel_established(ssl_tunnel_established), m_context(context)
+        {}
+        
+        void start_proxy_connect()
+        {
+            auto proxy = m_context->m_http_client->client_config().proxy();
+            auto proxy_uri = proxy.address();
+
+            utility::string_t proxy_host = proxy_uri.host();
+            int proxy_port = proxy_uri.port() == -1 ? 8080 : proxy_uri.port();
+
+            const auto &base_uri = m_context->m_http_client->base_uri();
+            const auto &host = base_uri.host();
+
+            std::ostream request_stream(&m_request);
+            request_stream.imbue(std::locale::classic());
+
+            request_stream << "CONNECT " << host << ":" << 443 << " HTTP/1.1" << CRLF;
+            request_stream << "Host: " << host << ":" << 443 << CRLF;
+            request_stream << "Proxy-Connection: Keep-Alive" << CRLF;
+
+            if(m_context->m_http_client->client_config().proxy().credentials().is_set())
+            {
+                request_stream << m_context->generate_basic_proxy_auth_header() << CRLF;
+            }
+
+            request_stream << CRLF;
+
+            m_context->m_timer.start();
+
+            tcp::resolver::query query(proxy_host, utility::conversions::print_string(proxy_port, std::locale::classic()));
+
+            auto client = std::static_pointer_cast<asio_client>(m_context->m_http_client);
+            client->m_resolver.async_resolve(query, boost::bind(&ssl_proxy_tunnel::handle_resolve, shared_from_this(), boost::asio::placeholders::error, boost::asio::placeholders::iterator));
+        }
+
+    private: 
+        void handle_resolve(const boost::system::error_code& ec, tcp::resolver::iterator endpoints)
+        {
+            if (ec)
+            {
+                m_context->report_error("Error resolving proxy address", ec, httpclient_errorcode_context::connect);
+            }
+            else
+            {
+                m_context->m_timer.reset();
+                auto endpoint = *endpoints;
+                m_context->m_connection->async_connect(endpoint, boost::bind(&ssl_proxy_tunnel::handle_tcp_connect, shared_from_this(), boost::asio::placeholders::error, ++endpoints));
+            }
+        }
+
+        void handle_tcp_connect(const boost::system::error_code& ec, tcp::resolver::iterator endpoints)
+        {
+            if (!ec)
+            {
+                m_context->m_timer.reset();
+                m_context->m_connection->async_write(m_request, boost::bind(&ssl_proxy_tunnel::handle_write_request, shared_from_this(), boost::asio::placeholders::error));
+            }
+            else if (endpoints == tcp::resolver::iterator())
+            {
+                m_context->report_error("Failed to connect to any resolved proxy endpoint", ec, httpclient_errorcode_context::connect);
+            }
+            else
+            {
+                m_context->m_timer.reset();
+                //// Replace the connection. This causes old connection object to go out of scope.
+                auto client = std::static_pointer_cast<asio_client>(m_context->m_http_client);
+                m_context->m_connection = client->m_pool.obtain();
+
+                auto endpoint = *endpoints;
+                m_context->m_connection->async_connect(endpoint, boost::bind(&ssl_proxy_tunnel::handle_tcp_connect, shared_from_this(), boost::asio::placeholders::error, ++endpoints));
+            }
+
+        }
+
+        void handle_write_request(const boost::system::error_code& err)
+        {
+            if (!err)
+            {
+                m_context->m_timer.reset();
+                m_context->m_connection->async_read_until(m_response, CRLF + CRLF, boost::bind(&ssl_proxy_tunnel::handle_status_line, shared_from_this(), boost::asio::placeholders::error));
+            }
+            else
+            {
+                m_context->report_error("Failed to send connect request to proxy.", err, httpclient_errorcode_context::writebody);
+            }
+	    }
+    
+        void handle_status_line(const boost::system::error_code& ec)
+        {
+            if (!ec)
+            {
+                m_context->m_timer.reset();
+                std::istream response_stream(&m_response);
+                response_stream.imbue(std::locale::classic());
+                std::string http_version;
+                response_stream >> http_version;
+                status_code status_code;
+                response_stream >> status_code;
+                
+                if (!response_stream || http_version.substr(0, 5) != "HTTP/")
+                {
+                    m_context->report_error("Invalid HTTP status line during proxy connection", ec, httpclient_errorcode_context::readheader);
+                    return;
+                }
+                
+                if (status_code != 200)
+                {
+                    utility::stringstream_t err_ss;
+                    err_ss << U("Expected a 200 response from proxy, received: ") << status_code;
+                    m_context->report_error(err_ss.str(), ec, httpclient_errorcode_context::readheader);
+                    return;
+                }
+                
+                m_context->m_connection->upgrade_to_ssl();
+                
+                m_ssl_tunnel_established(m_context);
+            }
+            else
+            {
+                // These errors tell if connection was closed.
+                const bool socket_was_closed((boost::asio::error::eof == ec)
+                    || (boost::asio::error::connection_reset == ec)
+                    || (boost::asio::error::connection_aborted == ec));
+                if (socket_was_closed && m_context->m_connection->is_reused())
+                {
+                    // Failed to write to socket because connection was already closed while it was in the pool.
+                    // close() here ensures socket is closed in a robust way and prevents the connection from being put to the pool again.
+                    m_context->m_connection->close();
+            
+                    // Create a new context and copy the request object, completion event and
+                    // cancellation registration to maintain the old state.
+                    // This also obtains a new connection from pool.
+                    auto new_ctx = m_context->create_request_context(m_context->m_http_client, m_context->m_request);
+                    new_ctx->m_request_completion = m_context->m_request_completion;
+                    new_ctx->m_cancellationRegistration = m_context->m_cancellationRegistration;
+            
+                    auto client = std::static_pointer_cast<asio_client>(m_context->m_http_client);
+                    // Resend the request using the new context.
+                    client->send_request(new_ctx);
+                }
+                else
+                {
+                    m_context->report_error("Failed to read HTTP status line from proxy", ec, httpclient_errorcode_context::readheader);
+                }
+            }
+        }
+    
+        std::function<void(std::shared_ptr<asio_context>)> m_ssl_tunnel_established;
+        std::shared_ptr<asio_context> m_context;
+    
+        boost::asio::streambuf m_request;
+        boost::asio::streambuf m_response;
+    };
+    
+    
+    enum class http_proxy_type
+    {
+        none,
+        http,
+        ssl_tunnel
+    };
 
     void start_request()
     {
@@ -371,94 +558,160 @@ public:
             request_context::report_error(make_error_code(std::errc::operation_canceled).value(), "Request canceled by user.");
             return;
         }
-
-        const auto &base_uri = m_http_client->base_uri();
-        auto encoded_resource = uri_builder(base_uri).append(m_request.relative_uri()).to_uri().resource().to_string();
-        if (encoded_resource == "")
+        
+        http_proxy_type proxy_type = http_proxy_type::none;
+        utility::string_t proxy_host;
+        int proxy_port = -1;
+        
+        // There is no support for auto-detection of proxies on non-windows platforms, it must be specified explicitly from the client code.
+        if (m_http_client->client_config().proxy().is_specified())
         {
-            encoded_resource = "/";
+            proxy_type = m_http_client->base_uri().scheme() == U("https") ? http_proxy_type::ssl_tunnel : http_proxy_type::http;
+            auto proxy = m_http_client->client_config().proxy();
+            auto proxy_uri = proxy.address();
+            proxy_port = proxy_uri.port() == -1 ? 8080 : proxy_uri.port();
+            proxy_host = proxy_uri.host();
         }
-
-        const auto &method = m_request.method();
-
-        // stop injection of headers via method
-        // resource should be ok, since it's been encoded
-        // and host won't resolve
-        if (!::web::http::details::validate_method(method))
+        
+        auto start_http_request_flow = [proxy_type, proxy_host, proxy_port](std::shared_ptr<asio_context> ctx)
         {
-            report_exception(http_exception("The method string is invalid."));
-            return;
-        }
-
-        const auto &host = base_uri.host();
-        std::ostream request_stream(&m_body_buf);
-        request_stream.imbue(std::locale::classic());
-
-        request_stream << method << " " << encoded_resource << " " << "HTTP/1.1" << CRLF << "Host: " << host;
-
-        int port = base_uri.port();
-        if (base_uri.is_port_default())
-        {
-            port = (m_connection->is_ssl() ? 443 : 80);
-        }
-        request_stream << ":" << port << CRLF;
-
-        // Extra request headers are constructed here.
-        utility::string_t extra_headers;
-
-        // Check user specified transfer-encoding.
-        std::string transferencoding;
-        if (m_request.headers().match(header_names::transfer_encoding, transferencoding) && transferencoding == "chunked")
-        {
-            m_needChunked = true;
-        }
-        else if (!m_request.headers().match(header_names::content_length, m_content_length))
-        {
-            // Stream without content length is the signal of requiring transfer encoding chunked.
-            if (m_request.body())
+            if (ctx->m_request._cancellation_token().is_canceled())
             {
-                m_needChunked = true;
-                extra_headers.append(header_names::transfer_encoding);
-                extra_headers.append(":chunked" + CRLF);
+                ctx->request_context::report_error(make_error_code(std::errc::operation_canceled).value(), "Request canceled by user.");
+                return;
             }
-        }
-
-        request_stream << flatten_http_headers(m_request.headers());
-        request_stream << extra_headers;
-        // Enforce HTTP connection keep alive (even for the old HTTP/1.0 protocol).
-        request_stream << "Connection: Keep-Alive" << CRLF << CRLF;
-
-        // Start connection timeout timer.
-        m_timer.start();
-
-        if (m_connection->is_reused())
+                
+            const auto &base_uri = ctx->m_http_client->base_uri();
+            const auto full_uri = uri_builder(base_uri).append(ctx->m_request.relative_uri()).to_uri();
+                
+            // For a normal http proxy, we need to specify the full request uri, otherwise just specify the resource
+            auto encoded_resource = proxy_type == http_proxy_type::http ? full_uri.to_string() : full_uri.resource().to_string();
+                
+            if (encoded_resource == "")
+            {
+                encoded_resource = "/";
+            }
+                
+            const auto &method = ctx->m_request.method();
+                
+            // stop injection of headers via method
+            // resource should be ok, since it's been encoded
+            // and host won't resolve
+            if (!::web::http::details::validate_method(method))
+            {
+                ctx->report_exception(http_exception("The method string is invalid."));
+                return;
+            }
+                
+            std::ostream request_stream(&ctx->m_body_buf);
+            request_stream.imbue(std::locale::classic());
+            const auto &host = base_uri.host();
+                
+            request_stream << method << " " << encoded_resource << " " << "HTTP/1.1" << CRLF;
+                
+            int port = base_uri.port();
+                
+            if (base_uri.is_port_default())
+            {
+                port = (ctx->m_connection->is_ssl() ? 443 : 80);
+            }
+                
+            // Add the Host header if user has not specified it explicitly
+            if (!ctx->m_request.headers().has(header_names::host))
+            {
+                request_stream << "Host: " << host << ":" << port << CRLF;
+            }
+                
+            // Extra request headers are constructed here.
+            utility::string_t extra_headers;
+                
+            // Add header for basic proxy authentication
+            if (proxy_type == http_proxy_type::http && ctx->m_http_client->client_config().proxy().credentials().is_set())
+            {
+                extra_headers.append(ctx->generate_basic_proxy_auth_header());
+            }
+                
+            // Check user specified transfer-encoding.
+            std::string transferencoding;
+            if (ctx->m_request.headers().match(header_names::transfer_encoding, transferencoding) && transferencoding == "chunked")
+            {
+                ctx->m_needChunked = true;
+            }
+            else if (!ctx->m_request.headers().match(header_names::content_length, ctx->m_content_length))
+            {
+                // Stream without content length is the signal of requiring transfer encoding chunked.
+                if (ctx->m_request.body())
+                {
+                    ctx->m_needChunked = true;
+                    extra_headers.append(header_names::transfer_encoding);
+                    extra_headers.append(":chunked" + CRLF);
+                }
+            }
+                
+            if (proxy_type == http_proxy_type::http)
+            {
+                extra_headers.append(header_names::cache_control);
+                extra_headers.append(": no-store, no-cache" + CRLF);
+                extra_headers.append(header_names::pragma);
+                extra_headers.append(": no-cache" + CRLF);
+            }
+                
+            request_stream << flatten_http_headers(ctx->m_request.headers());
+            request_stream << extra_headers;
+            // Enforce HTTP connection keep alive (even for the old HTTP/1.0 protocol).
+            request_stream << "Connection: Keep-Alive" << CRLF << CRLF;
+                
+            // Start connection timeout timer.
+            if (!ctx->m_timer.has_started())
+            {
+                ctx->m_timer.start();
+            }
+                
+            if (ctx->m_connection->is_reused() || proxy_type == http_proxy_type::ssl_tunnel)
+            {
+                // If socket is a reused connection or we're connected via an ssl-tunneling proxy, try to write the request directly. In both cases we have already established a tcp connection.
+                ctx->write_request();
+            }
+            else
+            {
+                // If the connection is new (unresolved and unconnected socket), then start async
+                // call to resolve first, leading eventually to request write.
+                    
+                // For normal http proxies, we want to connect directly to the proxy server. It will relay our request.
+                auto tcp_host = proxy_type == http_proxy_type::http ? proxy_host : host;
+                auto tcp_port = proxy_type == http_proxy_type::http ? proxy_port : port;
+                    
+                tcp::resolver::query query(tcp_host, utility::conversions::print_string(tcp_port, std::locale::classic()));
+                auto client = std::static_pointer_cast<asio_client>(ctx->m_http_client);
+                client->m_resolver.async_resolve(query, boost::bind(&asio_context::handle_resolve, ctx, boost::asio::placeholders::error, boost::asio::placeholders::iterator));
+            }
+        
+                // Register for notification on cancellation to abort this request.
+            if (ctx->m_request._cancellation_token() != pplx::cancellation_token::none())
+            {
+                // weak_ptr prevents lambda from taking shared ownership of the context.
+                // Otherwise context replacement in the handle_status_line() would leak the objects.
+                std::weak_ptr<asio_context> ctx_weak(ctx);
+                ctx->m_cancellationRegistration = ctx->m_request._cancellation_token().register_callback([ctx_weak]()
+                {
+                    if (auto ctx_lock = ctx_weak.lock())
+                    {
+                        // Shut down transmissions, close the socket and prevent connection from being pooled.
+                        ctx_lock->m_connection->close();
+                    }
+                });
+            }
+        };
+        
+        if (proxy_type == http_proxy_type::ssl_tunnel)
         {
-            // If socket is a reused connection, try to write the request directly.
-            write_request();
+            // The ssl_tunnel_proxy keeps the context alive and then calls back once the ssl tunnel is established via 'start_http_request_flow'
+            std::shared_ptr<ssl_proxy_tunnel> ssl_tunnel = std::make_shared<ssl_proxy_tunnel>(shared_from_this(), start_http_request_flow);
+            ssl_tunnel->start_proxy_connect();
         }
         else
         {
-            // If the connection is new (unresolved and unconnected socket), then start async
-            // call to resolve first, leading eventually to request write.
-            tcp::resolver::query query(host, utility::conversions::print_string(port, std::locale::classic()));
-            auto client = std::static_pointer_cast<asio_client>(m_http_client);
-            client->m_resolver.async_resolve(query, boost::bind(&asio_context::handle_resolve, shared_from_this(), boost::asio::placeholders::error, boost::asio::placeholders::iterator));
-        }
-
-        // Register for notification on cancellation to abort this request.
-        if(m_request._cancellation_token() != pplx::cancellation_token::none())
-        {
-            // weak_ptr prevents lambda from taking shared ownership of the context.
-            // Otherwise context replacement in the handle_status_line() would leak the objects.
-            std::weak_ptr<asio_context> ctx_weak(shared_from_this());
-            m_cancellationRegistration = m_request._cancellation_token().register_callback([ctx_weak]()
-            {
-                if (auto ctx_lock = ctx_weak.lock())
-                {
-                    // Shut down transmissions, close the socket and prevent connection from being pooled.
-                    ctx_lock->m_connection->close();
-                }
-            });
+            start_http_request_flow(shared_from_this());
         }
     }
 
@@ -477,6 +730,24 @@ public:
 
 private:
 
+    utility::string_t generate_basic_proxy_auth_header()
+    {
+        utility::string_t header;
+        
+        header.append(header_names::proxy_authorization);
+        header.append(": Basic ");
+        
+        auto credential_str = web::details::plaintext_string(new ::utility::string_t(m_http_client->client_config().proxy().credentials().username()));
+        credential_str->append(":");
+        credential_str->append(*m_http_client->client_config().proxy().credentials().decrypt());
+        
+        std::vector<unsigned char> credentials_buffer(credential_str->begin(), credential_str->end());
+        
+        header.append(utility::conversions::to_base64(credentials_buffer));
+        header.append(CRLF);
+        return header;
+    }
+
     void report_error(const utility::string_t &message, const boost::system::error_code &ec, httpclient_errorcode_context context = httpclient_errorcode_context::none)
     {
         // By default, errorcodeValue don't need to converted
@@ -519,11 +790,16 @@ private:
 
     void handle_connect(const boost::system::error_code& ec, tcp::resolver::iterator endpoints)
     {
+       
         m_timer.reset();
         if (!ec)
         {
             write_request();
         }
+        else if (ec.value() == boost::system::errc::operation_canceled)
+        {
+            request_context::report_error(ec.value(), "Request canceled by user.");
+        }
         else if (endpoints == tcp::resolver::iterator())
         {
             report_error("Failed to connect to any resolved endpoint", ec, httpclient_errorcode_context::connect);
@@ -561,6 +837,7 @@ private:
             const auto weakCtx = std::weak_ptr<asio_context>(shared_from_this());
             m_connection->async_handshake(boost::asio::ssl::stream_base::client,
                                           m_http_client->client_config(),
+                                          m_http_client->base_uri().host(),
                                           boost::bind(&asio_context::handle_handshake, shared_from_this(), boost::asio::placeholders::error),
 
                                           // Use a weak_ptr since the verify_callback is stored until the connection is destroyed.
@@ -640,6 +917,7 @@ private:
         }
     }
 
+
     void handle_write_chunked_body(const boost::system::error_code& ec)
     {
         if (ec)
@@ -705,7 +983,7 @@ private:
             // Reuse error handling.
             return handle_write_body(ec);
         }
-
+        
         m_timer.reset();
         const auto &progress = m_request._get_impl()->_progress_handler();
         if (progress)
@@ -786,7 +1064,7 @@ private:
             response_stream >> http_version;
             status_code status_code;
             response_stream >> status_code;
-
+            
             std::string status_message;
             std::getline(response_stream, status_message);
 
@@ -1114,6 +1392,8 @@ private:
 
         bool has_timedout() const { return m_state == timedout; }
 
+        bool has_started() const { return m_state == started; }
+
         void stop()
         {
             m_state = stopped;
@@ -1135,6 +1415,7 @@ private:
             }
         }
 
+    private:
         enum timer_state
         {
             created,
@@ -1164,6 +1445,8 @@ private:
 #endif
 };
 
+
+
 http_network_handler::http_network_handler(const uri &base_uri, const http_client_config &client_config) :
     m_http_client_impl(std::make_shared<asio_client>(base_uri, client_config))
 {}
